@@ -69,8 +69,8 @@ typedef struct {
         int   ignore_damage;
 } MagnifierOptions;
 
-static MagnifierOptions global_options = { ":0.0",
-					   ":0.0",
+static MagnifierOptions global_options = { NULL,
+					   NULL,
 					   "default",
 					   "none",
 					   2.0,
@@ -379,6 +379,16 @@ main (int argc, char** argv)
 	  gchar *string;
 	  string = g_strconcat ("DISPLAY=", global_options.target_display, NULL);
 	  putenv (string);
+  } else {
+		  global_options.target_display = getenv ("DISPLAY");
+		  if (!global_options.target_display) {
+			  fprintf (stderr, _("Can't open display, DISPLAY is not set"));
+			  exit (1);
+		  }
+  }  
+
+  if (!global_options.source_display) {
+		  global_options.source_display = global_options.target_display;
   }
 
   if (global_options.timing_pan_rate && global_options.timing_iterations == 0)
