@@ -143,8 +143,6 @@ static int  zoom_region_process_updates (gpointer data);
 static void zoom_region_paint (ZoomRegion *zoom_region, GdkRectangle *rect);
 static void zoom_region_paint_pixmap (ZoomRegion *zoom_region, GdkRectangle *rect);
 static int  zoom_region_update_pointer_timeout (gpointer data);
-static void zoom_region_recompute_exposed_viewport (ZoomRegion *zoom_region);
-
 static GdkRectangle zoom_region_rect_from_bounds (ZoomRegion *zoom_region,
 						  const GNOME_Magnifier_RectBounds *bounds);
 static ZoomRegionPixmapCreationError zoom_region_create_pixmap (ZoomRegion *zoom_region);
@@ -657,6 +655,40 @@ zoom_region_rect_from_bounds (ZoomRegion *zoom_region,
 
 /** **************  **/
 
+static CORBA_boolean
+zoom_region_update_scale (ZoomRegion *zoom_region, gdouble x, gdouble y)
+{
+	gdouble x_old = zoom_region->xscale;
+	gdouble y_old = zoom_region->yscale;
+
+	zoom_region->xscale = x;
+	zoom_region->yscale = y;
+
+	if (zoom_region->priv->scaled_pixbuf)
+		g_object_unref (zoom_region->priv->scaled_pixbuf);
+	zoom_region->priv->scaled_pixbuf =
+		gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, (zoom_region->priv->source_area.x2 - zoom_region->priv->source_area.x1) * zoom_region->xscale + 1, (zoom_region->priv->source_area.y2 - zoom_region->priv->source_area.y1) * zoom_region->yscale + 1);
+
+	if (zoom_region->priv->pixmap)
+		g_object_unref (zoom_region->priv->pixmap);
+
+	if (zoom_region_create_pixmap (zoom_region) ==
+	    ZOOM_REGION_ERROR_TOO_BIG) {
+		zoom_region->xscale = x_old;
+		zoom_region->yscale = y_old;
+		zoom_region_create_pixmap (zoom_region);
+		g_object_unref (zoom_region->priv->scaled_pixbuf);
+
+		/* only create a scaled image big enough for the target
+		 * display, for now */
+		zoom_region->priv->scaled_pixbuf = gdk_pixbuf_new (
+			GDK_COLORSPACE_RGB, FALSE, 8, (zoom_region->priv->source_area.x2 - zoom_region->priv->source_area.x1) * zoom_region->xscale + 1, (zoom_region->priv->source_area.y2 - zoom_region->priv->source_area.y1) * zoom_region->yscale + 1);
+
+		return CORBA_FALSE;
+	}
+	return CORBA_TRUE;
+}
+
 static void
 zoom_region_queue_update (ZoomRegion *zoom_region,
 			  const GdkRectangle update_rect)
@@ -695,7 +727,7 @@ zoom_region_update_current (ZoomRegion *zoom_region)
 			zoom_region_update (zoom_region,
 					    zoom_region_source_rect_from_view_bounds (
 						    zoom_region,
-						    &zoom_region->priv->exposed_viewport));
+						    &zoom_region->viewport));
 	}
 }
 
@@ -1013,37 +1045,29 @@ zoom_region_coalesce_updates (ZoomRegion *zoom_region)
 
 
 static void
-zoom_region_paint_border (ZoomRegion *zoom_region,
-			  GdkRectangle *area)
+zoom_region_paint_border (ZoomRegion *zoom_region)
 {
 	GdkColor color;
 
 #ifdef ZOOM_REGION_DEBUG
 	g_assert (zoom_region->alive);
 #endif
-	if ((zoom_region->border_size > 0) && (zoom_region->priv->w->window))
-	{
-		if (!zoom_region->priv->border_gc)
-		{
-			zoom_region->priv->border_gc = gdk_gc_new (zoom_region->priv->w->window);
-			color.red = (zoom_region->border_color & 0xFF0000) >> 8;
-			color.green = (zoom_region->border_color & 0xFF00);
-			color.blue = (zoom_region->border_color & 0xFF) << 8;
+	if ((zoom_region->border_size > 0) &&
+	    (zoom_region->priv->border->window)) {
+		color.red = (((zoom_region->border_color & 0xFF0000) >> 16) *
+			     65535) / 255;
+		color.green = (((zoom_region->border_color & 0xFF00) >> 8) *
+			       65535) / 255;
+		color.blue = ((zoom_region->border_color & 0xFF) * 65535) /
+			255;
+
 #ifdef DEBUG_BORDER
-			fprintf (stderr, "border color triple RGB=%d|%d|%d",
-				 color.red, color.green, color.blue);
+		fprintf (stderr, "border color triple RGB=%d|%d|%d\n",
+			 color.red, color.green, color.blue);
 #endif
-			gdk_colormap_alloc_color (gdk_drawable_get_colormap (zoom_region->priv->w->window), 
-						  &color, TRUE, TRUE);
-			gdk_gc_set_foreground (zoom_region->priv->border_gc, &color);
-		}
-		gdk_draw_rectangle (zoom_region->priv->w->window,
-				    zoom_region->priv->border_gc,
-				    TRUE,
-				    area->x,
-				    area->y,
-				    area->width,
-				    area->height);
+
+		gtk_widget_modify_bg (zoom_region->priv->border,
+				      GTK_STATE_NORMAL, &color);
 	}
 }
 
@@ -1089,7 +1113,6 @@ zoom_region_paint (ZoomRegion *zoom_region,
 #endif
 	DEBUG_RECT ("painting (clipped)", *area);
 	paint_area = zoom_region_clip_to_window (zoom_region, *area);
-	zoom_region_paint_border (zoom_region, area);
 	zoom_region_paint_pixmap (zoom_region, &paint_area);
 	zoom_region_paint_cursor (zoom_region, &paint_area);
 	zoom_region_paint_crosswire_cursor (zoom_region, &paint_area);
@@ -1120,31 +1143,22 @@ zoom_region_create_pixmap (ZoomRegion *zoom_region)
 			return ZOOM_REGION_ERROR_TOO_BIG;
 		}
 
-		zoom_region_recompute_exposed_viewport (zoom_region);
-#ifdef ZOOM_REGION_DEBUG
-		g_message ("create-pixmap-update: %d,%d - %d,%d",
-			   zoom_region->priv->exposed_viewport.x1,
-			   zoom_region->priv->exposed_viewport.y1,
-			   zoom_region->priv->exposed_viewport.x2,
-			   zoom_region->priv->exposed_viewport.y2);
-#endif
+		DEBUG_RECT("viewport", zoom_region_source_rect_from_view_bounds
+			   (zoom_region, &zoom_region->viewport));
+		DEBUG_RECT("source", zoom_region_rect_from_bounds
+			   (zoom_region, &((Magnifier*)zoom_region->priv->parent)->source_bounds));
 
-			    DEBUG_RECT("viewport", zoom_region_source_rect_from_view_bounds
-					    (zoom_region, &zoom_region->priv->exposed_viewport));
-			    DEBUG_RECT("source", zoom_region_rect_from_bounds
-					    (zoom_region, &((Magnifier*)zoom_region->priv->parent)->source_bounds));
-
-			    zoom_region_update (zoom_region,
+		zoom_region_update (zoom_region,
 /*				    zoom_region_source_rect_from_view_bounds (
-					    zoom_region,
-					    &zoom_region->priv->exposed_viewport));
+				    zoom_region,
+				    &zoom_region->viewport));
 */
 				    zoom_region_rect_from_bounds 
 				    (zoom_region, 
 				     &((Magnifier *)zoom_region->priv->parent)->source_bounds));
 		return ZOOM_REGION_ERROR_NONE;
 	}
-
+	
 	return ZOOM_REGION_ERROR_NO_TARGET_DRAWABLE;
 }
 
@@ -1191,8 +1205,10 @@ zoom_region_update_cursor (ZoomRegion *zoom_region, int dx, int dy,
 	zoom_region->priv->last_drawn_crosswire_pos.y += dy;
 	zoom_region_paint_cursor (zoom_region, clip_rect);
 	zoom_region_paint_crosswire_cursor (zoom_region, clip_rect);
-	if (GTK_IS_WIDGET (zoom_region->priv->w) && GDK_IS_WINDOW (zoom_region->priv->w->window))
-	    gdk_display_sync (gdk_drawable_get_display (zoom_region->priv->w->window));
+	if (GTK_IS_WIDGET (zoom_region->priv->w) &&
+	    GDK_IS_WINDOW (zoom_region->priv->w->window))
+		gdk_display_sync (gdk_drawable_get_display (
+					  zoom_region->priv->w->window));
 }
 
 static gboolean
@@ -1273,7 +1289,8 @@ zoom_region_scroll_fast (ZoomRegion *zoom_region, int dx, int dy,
 	zoom_region_paint_crosswire_cursor (zoom_region, scroll_rect);
 	gdk_window_process_updates (window, FALSE);
 	/* sync reduces cursor flicker, but slows things down */
-	if (zoom_region->smooth_scroll_policy > GNOME_Magnifier_ZoomRegion_SCROLL_FASTEST)
+	if (zoom_region->smooth_scroll_policy >
+	    GNOME_Magnifier_ZoomRegion_SCROLL_FASTEST)
 		gdk_display_sync (gdk_drawable_get_display (window)); 
 }
 
@@ -1357,19 +1374,6 @@ zoom_region_scroll (ZoomRegion *zoom_region, int dx, int dy)
 	}
 }
 
-static void
-zoom_region_recompute_exposed_viewport (ZoomRegion *zoom_region)
-{
-	zoom_region->priv->exposed_viewport.x1 = zoom_region->viewport.x1
-		+ zoom_region->border_size;
-	zoom_region->priv->exposed_viewport.y1 = zoom_region->viewport.y1
-		+ zoom_region->border_size;;
-	zoom_region->priv->exposed_viewport.x2 = zoom_region->viewport.x2
-		- zoom_region->border_size;;
-	zoom_region->priv->exposed_viewport.y2 = zoom_region->viewport.y2
-		- zoom_region->border_size;
-}
-
 static void
 zoom_region_recompute_exposed_bounds (ZoomRegion *zoom_region)
 {
@@ -1605,12 +1609,12 @@ zoom_region_get_source_subwindow (ZoomRegion *zoom_region,
 						    gdk_visual_get_system (),
 						    width,
 						    height);
-			
+
 			for (i = 0; i < width; ++i)
 				for (j = 0; j < height; ++j)
 					gdk_image_put_pixel (test_image, i, j, i*j);
 
-			zoom_region->priv->source_drawable = gdk_pixmap_new (NULL, width, height, 24);
+			zoom_region->priv->source_drawable = gdk_pixmap_new (zoom_region->priv->w->window, width, height, -1);
 
 			if (zoom_region->priv->default_gc == NULL)
 				zoom_region->priv->default_gc = gdk_gc_new(zoom_region->priv->w->window);
@@ -1783,7 +1787,8 @@ zoom_region_update (ZoomRegion *zoom_region,
 		GdkRectangle source_rect = zoom_region_update_pixmap (zoom_region, update_rect, &paint_rect);
 		if (paint_rect.x != 0 || paint_rect.y != 0 ||
 		    paint_rect.width != 0 || paint_rect.height != 0) {
-			gdk_window_begin_paint_rect (zoom_region->priv->w->window, &paint_rect);
+			gdk_window_begin_paint_rect (
+				zoom_region->priv->w->window, &paint_rect);
 			zoom_region_paint (zoom_region, &paint_rect);
 			gdk_window_end_paint (zoom_region->priv->w->window);
 		}
@@ -1833,34 +1838,40 @@ zoom_region_init_window (ZoomRegion *zoom_region)
 	parent = GTK_FIXED (
 		((Magnifier *)zoom_region->priv->parent)->priv->canvas);
 	zoomer = gtk_drawing_area_new ();
-	border = gtk_fixed_new ();
+	border = gtk_drawing_area_new ();
 	zoom_region->priv->border = border;
 	zoom_region->priv->w = zoomer;
 
 #ifdef ZOOM_REGION_DEBUG
 	g_assert (zoom_region->alive);
 #endif
+	gtk_widget_set_size_request (GTK_WIDGET (border),
+				     zoom_region->viewport.x2 -
+				     zoom_region->viewport.x1,
+				     zoom_region->viewport.y2 -
+				     zoom_region->viewport.y1);
 	gtk_widget_set_size_request (GTK_WIDGET (zoomer),
 				     zoom_region->viewport.x2 -
-				     zoom_region->viewport.x1 - zoom_region->border_size * 2,
+				     zoom_region->viewport.x1 -
+				     zoom_region->border_size * 2,
 				     zoom_region->viewport.y2 -
-				     zoom_region->viewport.y1 - zoom_region->border_size * 2);
+				     zoom_region->viewport.y1 -
+				     zoom_region->border_size * 2);
 	gtk_fixed_put (parent, border,
 		       zoom_region->viewport.x1,
 		       zoom_region->viewport.y1);
-	gtk_fixed_put (GTK_FIXED (border), zoomer,
-		       zoom_region->border_size,
-		       zoom_region->border_size);
+	gtk_fixed_put (parent, zoomer,
+		       zoom_region->viewport.x1 + zoom_region->border_size,
+		       zoom_region->viewport.y1 + zoom_region->border_size);
 	gtk_widget_show (GTK_WIDGET (border));
 	gtk_widget_show (GTK_WIDGET (zoomer));
 	gtk_widget_show (GTK_WIDGET (parent));
 	zoom_region->priv->expose_handler_id =
 		g_signal_connect (G_OBJECT (zoom_region->priv->w),
-			    "expose_event",
-			    G_CALLBACK (zoom_region_expose_handler),
-			    zoom_region);
+				  "expose_event",
+				  G_CALLBACK (zoom_region_expose_handler),
+				  zoom_region);
 	DBG(fprintf (stderr, "New window created\n"));
-	gtk_widget_show (GTK_WIDGET (zoom_region->priv->w));
 }
 
 static int
@@ -1904,10 +1915,10 @@ timing_report(ZoomRegion *zoom_region)
 	int width, height, x, y;
 
 	if (timing_test) {
-		width = (zoom_region->priv->exposed_viewport.x2 -
-			zoom_region->priv->exposed_viewport.x1) / zoom_region->xscale;
-		height = (zoom_region->priv->exposed_viewport.y2 -
-			zoom_region->priv->exposed_viewport.y1) / zoom_region->yscale;
+		width = (zoom_region->viewport.x2 -
+			zoom_region->viewport.x1) / zoom_region->xscale;
+		height = (zoom_region->viewport.y2 -
+			zoom_region->viewport.y1) / zoom_region->yscale;
 
 		frame_avg = mag_timing.frame_total / mag_timing.num_frame_samples;
 
@@ -2117,10 +2128,10 @@ zoom_region_align (ZoomRegion *zoom_region)
 	g_idle_add_full (GDK_PRIORITY_REDRAW + 1,
 		gdk_timing_idle, zoom_region, NULL);
 
-	width = (zoom_region->priv->exposed_viewport.x2 -
-		zoom_region->priv->exposed_viewport.x1) / zoom_region->xscale;
-	height = (zoom_region->priv->exposed_viewport.y2 -
-		zoom_region->priv->exposed_viewport.y1) / zoom_region->yscale;
+	width = (zoom_region->viewport.x2 -
+		zoom_region->viewport.x1) / zoom_region->xscale;
+	height = (zoom_region->viewport.y2 -
+		zoom_region->viewport.y1) / zoom_region->yscale;
 
 	switch (zoom_region->x_align_policy) {
 	case GNOME_Magnifier_ZoomRegion_ALIGN_MAX:
@@ -2156,13 +2167,18 @@ zoom_region_set_viewport (ZoomRegion *zoom_region,
 #ifdef ZOOM_REGION_DEBUG
 	g_assert (zoom_region->alive);
 #endif
+	if (zoom_region->viewport.x1 == viewport->x1 &&
+	    zoom_region->viewport.y1 == viewport->y1 &&
+	    zoom_region->viewport.x2 == viewport->x2 &&
+	    zoom_region->viewport.y2 == viewport->y2) {
+		return;
+	}
 	zoom_region->viewport = *viewport;
 #ifdef DEBUG
 	fprintf (stderr, "Setting viewport %d,%d - %d,%d\n",
 		 (int) viewport->x1, (int) viewport->y1,
 		 (int) viewport->x2, (int) viewport->y2);
 #endif
-	zoom_region_recompute_exposed_viewport (zoom_region);
 	zoom_region_align (zoom_region);
 	if (!zoom_region->priv->w) {
 		zoom_region_init_window (zoom_region);
@@ -2176,11 +2192,23 @@ zoom_region_set_viewport (ZoomRegion *zoom_region,
 				zoom_region->priv->border,
 				zoom_region->viewport.x1,
 				zoom_region->viewport.y1);
+		gtk_fixed_move (fixed,
+				zoom_region->priv->w,
+				zoom_region->viewport.x1 +
+				zoom_region->border_size,
+				zoom_region->viewport.y1 +
+				zoom_region->border_size);
+		gtk_widget_set_size_request (
+			GTK_WIDGET (zoom_region->priv->border),
+			zoom_region->viewport.x2 - zoom_region->viewport.x1,
+			zoom_region->viewport.y2 - zoom_region->viewport.y1);
 		gtk_widget_set_size_request (GTK_WIDGET (zoom_region->priv->w),
 					     zoom_region->viewport.x2 -
-					     zoom_region->viewport.x1,
+					     zoom_region->viewport.x1 -
+					     zoom_region->border_size * 2,
 					     zoom_region->viewport.y2 -
-					     zoom_region->viewport.y1);
+					     zoom_region->viewport.y1 -
+					     zoom_region->border_size * 2);
 		CORBA_exception_init (&ev);
 		properties = 
 			GNOME_Magnifier_Magnifier_getProperties(
@@ -2350,8 +2378,10 @@ zoom_region_set_property (BonoboPropertyBag *bag,
 		break;
 	case ZOOM_REGION_TESTPATTERN_PROP:
 		zoom_region->priv->test = BONOBO_ARG_GET_BOOLEAN (arg);
-		if (zoom_region->priv->source_drawable)
+		if (zoom_region->priv->source_drawable) {
 			g_object_unref (zoom_region->priv->source_drawable);
+			zoom_region->priv->source_drawable = NULL;
+		}
 		zoom_region_update_current (zoom_region);
 		break;
 	case ZOOM_REGION_CONTRASTR_PROP:
@@ -2367,25 +2397,39 @@ zoom_region_set_property (BonoboPropertyBag *bag,
 		zoom_region_update_current (zoom_region);
 		break;
 	case ZOOM_REGION_XSCALE_PROP:
-		zoom_region->xscale = BONOBO_ARG_GET_FLOAT (arg);
+		(void) zoom_region_update_scale (zoom_region,
+						 BONOBO_ARG_GET_FLOAT (arg),
+						 zoom_region->yscale);
 		zoom_region_update_current (zoom_region);
 		break;
 	case ZOOM_REGION_YSCALE_PROP:
-		zoom_region->yscale = BONOBO_ARG_GET_FLOAT (arg);
+		(void) zoom_region_update_scale (zoom_region,
+						 zoom_region->xscale,
+						 BONOBO_ARG_GET_FLOAT (arg));
 		zoom_region_update_current (zoom_region);
 		break;
 	case ZOOM_REGION_BORDERSIZE_PROP:
 		zoom_region->border_size = BONOBO_ARG_GET_LONG (arg);
-		zoom_region_recompute_exposed_viewport (zoom_region);
-		zoom_region_update_current (zoom_region);
+		gtk_widget_set_size_request (
+			GTK_WIDGET (zoom_region->priv->border),
+			zoom_region->viewport.x2 -
+			zoom_region->viewport.x1,
+			zoom_region->viewport.y2 -
+			zoom_region->viewport.y1);
+		gtk_widget_set_size_request (GTK_WIDGET (zoom_region->priv->w),
+					     zoom_region->viewport.x2 -
+					     zoom_region->viewport.x1 -
+					     zoom_region->border_size * 2,
+					     zoom_region->viewport.y2 -
+					     zoom_region->viewport.y1 -
+					     zoom_region->border_size * 2);
+		gtk_fixed_move (GTK_FIXED (((Magnifier *)zoom_region->priv->parent)->priv->canvas), zoom_region->priv->border, zoom_region->viewport.x1, zoom_region->viewport.y1);
+		gtk_fixed_move (GTK_FIXED (((Magnifier *)zoom_region->priv->parent)->priv->canvas), zoom_region->priv->w, zoom_region->viewport.x1 + zoom_region->border_size, zoom_region->viewport.y1 + zoom_region->border_size);
 		break;
 	case ZOOM_REGION_BORDERCOLOR_PROP:
 		zoom_region->border_color =
 			BONOBO_ARG_GET_LONG (arg);
-		if (zoom_region->priv->border_gc)
-			g_object_unref (zoom_region->priv->border_gc);
-		zoom_region->priv->border_gc = NULL;
-		zoom_region_update_current (zoom_region);
+		zoom_region_paint_border (zoom_region);
 		break;
 	case ZOOM_REGION_XALIGN_PROP:
 		zoom_region->x_align_policy = BONOBO_ARG_GET_INT (arg);
@@ -2455,8 +2499,8 @@ zoom_region_pan_test (gpointer data)
 		gdk_display_get_screen (magnifier->source_display,
 		 magnifier->source_screen_num));
 
-	height = (zoom_region->priv->exposed_viewport.y2 -
-		zoom_region->priv->exposed_viewport.y1) / zoom_region->yscale;
+	height = (zoom_region->viewport.y2 -
+		zoom_region->viewport.y1) / zoom_region->yscale;
 
 	roi.x1 = zoom_region->roi.x1;
 	roi.x2 = zoom_region->roi.x2;
@@ -2627,6 +2671,13 @@ impl_zoom_region_set_roi (PortableServer_Servant servant,
 	DBG (fprintf (stderr, "Set ROI: \t%d,%d %d,%d\n", 
 		      bounds->x1, bounds->y1, bounds->x2, bounds->y2));
 
+	if ((zoom_region->roi.x1 == bounds->x1) &&
+	    (zoom_region->roi.x2 == bounds->x2) &&
+	    (zoom_region->roi.y1 == bounds->y1) &&
+	    (zoom_region->roi.y2 == bounds->y2)) {
+	    return;
+	}
+
 	/* if these bounds are clearly bogus, warn and ignore */
 	if (!bounds || (bounds->x2 <= bounds->x1)
 	    || (bounds->y2 < bounds->y1) || 
@@ -2696,10 +2747,13 @@ impl_zoom_region_set_mag_factor (PortableServer_Servant servant,
 	g_assert (zoom_region->alive);
 #endif
 	CORBA_any *any;
-	double xs_old = zoom_region->xscale;
-	double ys_old = zoom_region->yscale;
 	CORBA_boolean retval = CORBA_TRUE;
 
+	if ((zoom_region->xscale == mag_factor_x) &&
+	    (zoom_region->yscale == mag_factor_y)) {
+		return retval;
+	}
+
 	/* TODO: assert that parent is magnifier object */
 	Bonobo_PropertyBag properties =
 		GNOME_Magnifier_Magnifier_getProperties(
@@ -2713,38 +2767,9 @@ impl_zoom_region_set_mag_factor (PortableServer_Servant servant,
 	else
 		retval = CORBA_FALSE;
 
-	zoom_region->xscale = mag_factor_x;
-	zoom_region->yscale = mag_factor_y;
-
-	if (zoom_region->priv->scaled_pixbuf)
-		g_object_unref (zoom_region->priv->scaled_pixbuf);
-
-	zoom_region->priv->scaled_pixbuf = gdk_pixbuf_new (
-		GDK_COLORSPACE_RGB, FALSE, 8,
-		(zoom_region->priv->source_area.x2 -
-		zoom_region->priv->source_area.x1) * zoom_region->xscale + 1,
-		(zoom_region->priv->source_area.y2 -
-		zoom_region->priv->source_area.y1) * zoom_region->yscale + 1);
-
-	if (zoom_region->priv->pixmap) {
-		g_object_unref (zoom_region->priv->pixmap);
-		zoom_region->priv->pixmap = NULL;
-	}
-	if (zoom_region_create_pixmap (zoom_region) == ZOOM_REGION_ERROR_TOO_BIG) {
-		zoom_region->xscale = xs_old;
-		zoom_region->yscale = ys_old;
-		zoom_region_create_pixmap (zoom_region);
-		g_object_unref (zoom_region->priv->scaled_pixbuf);
-
-		/* only create a scaled image big enough for the target display, for now */
-		zoom_region->priv->scaled_pixbuf = gdk_pixbuf_new (
-			GDK_COLORSPACE_RGB, FALSE, 8,
-			(zoom_region->priv->source_area.x2 -
-			zoom_region->priv->source_area.x1) * zoom_region->xscale + 1,
-			(zoom_region->priv->source_area.y2 -
-			zoom_region->priv->source_area.y1) * zoom_region->yscale + 1);
-		retval = CORBA_FALSE;
-	}
+	retval = zoom_region_update_scale (zoom_region,
+					   mag_factor_x, mag_factor_y);
+	
 	zoom_region_update_current (zoom_region);
 	zoom_region_sync (zoom_region);
 
@@ -3190,7 +3215,6 @@ zoom_region_private_init (ZoomRegionPrivate *priv)
 	priv->pixmap = NULL;
 	priv->cursor_backing_rect = rect;
 	priv->cursor_backing_pixels = NULL;
-	priv->border_gc = NULL;
 	priv->gdk_interp_type = GDK_INTERP_NEAREST;
 	priv->expose_handler_id = 0;
 	priv->test = FALSE;
@@ -3199,7 +3223,6 @@ zoom_region_private_init (ZoomRegionPrivate *priv)
 	priv->last_drawn_crosswire_pos.x = 0;
 	priv->last_drawn_crosswire_pos.y = 0;
 	priv->exposed_bounds = rectbounds;
-	priv->exposed_viewport = rectbounds;
 	priv->source_area = rectbounds;
 	priv->update_pointer_id = 0;
 	priv->update_handler_id = 0;
@@ -3264,12 +3287,9 @@ zoom_region_finalize (GObject *region)
 		zoom_region->priv->q = NULL;
 	}
 	if (GTK_IS_WIDGET (zoom_region->priv->w))
-		gtk_container_remove (GTK_CONTAINER (zoom_region->priv->border),
-				      GTK_WIDGET (zoom_region->priv->w));
+		gtk_container_remove (GTK_CONTAINER (((Magnifier *) zoom_region->priv->parent)->priv->canvas), GTK_WIDGET (zoom_region->priv->w));
 	if (GTK_IS_WIDGET (zoom_region->priv->border))
-		gtk_container_remove (GTK_CONTAINER (((Magnifier *) 
-						      zoom_region->priv->parent)->priv->canvas),
-				      GTK_WIDGET (zoom_region->priv->border));
+		gtk_container_remove (GTK_CONTAINER (((Magnifier *) zoom_region->priv->parent)->priv->canvas), GTK_WIDGET (zoom_region->priv->border));
 	if (zoom_region->priv->source_pixbuf_cache) 
 	    g_object_unref (zoom_region->priv->source_pixbuf_cache);
 	if (zoom_region->priv->scaled_pixbuf) 
@@ -3280,8 +3300,6 @@ zoom_region_finalize (GObject *region)
 	zoom_region->priv->parent = NULL;
 	if (zoom_region->priv->cursor_backing_pixels) 
 	    g_object_unref (zoom_region->priv->cursor_backing_pixels);
-	if (zoom_region->priv->border_gc) 
-	    g_object_unref (zoom_region->priv->border_gc);
 	g_free (zoom_region->priv);
 	zoom_region->priv = NULL;
 #ifdef ZOOM_REGION_DEBUG
